import json
import urllib.parse

import arrow
import requests


def get_hc_settings():
    return {
        'base_url': 'https://example.zendesk.com',
        'credentials': ('jdoe@zendesk.com', 'some_pwd'),
        'agent_id': 5432154321
    }


def get_new_posts(topic_id, start_time):
    print('community.get_new_posts() starting')
    if start_time:                                      # if a start time is defined
        start_time = arrow.get(start_time)
    else:
        start_time = arrow.utcnow().shift(hours=-1)     # one hour ago for first request

    hc = get_hc_settings()
    url = f'{hc["base_url"]}/api/v2/community/topics/{topic_id}/posts.json'
    headers = {'Content-Type': 'application/json'}

    # get new posts
    new_posts = []
    while url:
        response = requests.get(url, auth=hc['credentials'], headers=headers)
        if response.status_code != 200:
            return {'error': {'status_code': response.status_code, 'text': response.text}}
        data = response.json()
        url = data['next_page']
        for post in data['posts']:
            created_at_time = arrow.get(post['created_at'])
            if created_at_time > start_time:
                new_posts.append(post)
            else:                   # no more new posts
                url = None          # stop paginating
                break               # exit for loop

    # reformat data for response
    external_resources = []
    for post in new_posts:
        resource = {
            'external_id': str(post['id']),
            'message': post['title'],
            'html_message': post['details'],
            'created_at': post['created_at'],
            'author': {'external_id': str(post['author_id']), 'name': 'community user'}
        }
        external_resources.append(resource)

    # get next start time
    if external_resources:
        created_at_times = []
        for resource in external_resources:
            created_at_times.append(arrow.get(resource['created_at']))
        start_time = max(created_at_times)   # get most recent datetime in list

    # add start_time to state parameter
    state = {'start_time': start_time.isoformat()}  # convert datetime to ISO 8601 string
    state = urllib.parse.quote(json.dumps(state))   # convert dict to JSON, then URL encode

    return {
        'external_resources': external_resources,
        'state': state
    }


def create_post_comment(post_id, comment):
    print('community.create_post_comment() starting')
    hc = get_hc_settings()
    url = f'{hc["base_url"]}/api/v2/community/posts/{post_id}/comments.json'
    data = {'comment': {'body': comment, 'author_id': hc['agent_id']}}
    headers = {'Content-Type': 'application/json', 'Accept': 'application/json'}

    response = requests.post(url, json=data, auth=hc['credentials'], headers=headers)
    if response.status_code != 201:
        print(f'Failed to create post comment with error {response.status_code}: {response.text}')
        return {'error': response.status_code}
    comment = response.json()['comment']
    return {'external_id': str(comment['id']), 'allow_channelback': False}
