import os
import json
import urllib.parse
from pathlib import Path

from bottle import route, run, request, response, template
import community


@route('/', method='GET')
def show_home():
    return '<p>Integration service for a Zendesk Support community channel</p>'


@route('/channels/community/pull', method='POST')
def pull_new_posts():
    print('service.pull_new_posts() starting')

    # get topic id
    metadata = request.forms['metadata']        # will always be set in admin ui
    metadata = urllib.parse.unquote(metadata)   # URL-decode the JSON value
    metadata = json.loads(metadata)             # convert decoded JSON to dict
    topic_id = metadata['topic_id']

    # get start time
    start_time = ''
    if request.forms['state']:          # if state has been defined in previous pull request
        state = urllib.parse.unquote(request.forms['state'])     # URL-decode the JSON value
        state = json.loads(state)       # convert decoded JSON to dict
        start_time = state['start_time']

    # get new community posts
    new_posts = community.get_new_posts(topic_id, start_time)
    if 'error' in new_posts:
        print(new_posts['error']['text'])
    else:
        response.status = 201
        response.headers['Content-Type'] = 'application/json'
        return json.dumps(new_posts)


@route('/channels/community/channelback', method='POST')
def channelback_ticket_comment():
    print('service.channelback_ticket_comment() starting')
    post_id = request.forms['parent_id']
    comment = request.forms['message']

    external_id = community.create_post_comment(post_id, comment)
    if 'error' in external_id:
        response.status = 500
        return
    response.status = 200
    response.headers['Content-Type'] = 'application/json'
    return json.dumps(external_id)


@route('/channels/community/admin_ui', method='POST')
def show_admin_ui():
    print('service.show_admin_ui() starting')
    return_url = request.forms['return_url']
    name = request.forms['name']
    if not name:
        name = 'Help Center community channel'

    topic_id = ''
    if 'metadata' in request.forms and request.forms['metadata']:   # metadata not included in initial request
        metadata = request.forms['metadata']
        metadata = urllib.parse.unquote(metadata)   # URL-decode the JSON value
        metadata = json.loads(metadata)             # convert decoded JSON to dict
        topic_id = metadata['topic_id']

    data = {
        'name': name,
        'topic_id': topic_id,
        'return_url': return_url
    }
    return template('admin', data=data)


@route('/channels/community/settings', method='POST')
def process_admin_settings():
    print('service.process_admin_settings() starting')
    topic_id = request.forms['topic_id']
    if not topic_id or not topic_id.isdigit():
        return template('admin', data={'topic_id': 'invalid'})

    # set metadata
    metadata = {'topic_id': topic_id}
    metadata = urllib.parse.quote(json.dumps(metadata))  # convert dict to JSON, then URL encode

    data = {
        'return_url': request.forms['return_url'],
        'name': request.forms['name'],
        'metadata': metadata
    }
    return template('admin_callback', data=data)


@route('/channels/community/manifest')
def serve_manifest():
    print('service.serve_manifest() starting')
    file = Path('integration_manifest.json')
    with file.open(mode='r') as f:
        manifest = json.load(f)
    response.headers['Content-Type'] = 'application/json'
    return manifest


print('web app starting')
if os.environ.get('APP_LOCATION') == 'heroku':
    run(host="0.0.0.0", port=int(os.environ.get("PORT", 5000)))
else:
    run(debug=True)
